(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is TurboPower OnGuard
 *
 * ***** END LICENSE BLOCK ***** *)

{**********************************************************************
 Package pl_OnGuard.pkg
 This unit is part of CodeTyphon Project (http://www.pilotlogic.com/)
Contributed by Shaun Rumbell
***********************************************************************}

unit ogwmi;

interface

{$mode delphi}{$H+}

type TID_Type = (BIOS, DISK, MOBO, OS,USER);

//-------------------------------------------------------------
// Function Wrapper around the windows WMI interface.
//
// Parameters:
//  ID -The type of ID to retrieve (BIOS, Disk Motherboard Operating System or User).
//  OutBuf -Variable length string with the requested information.
//
//
// Notes:
//  1. Outbuf will contain the string concatenation of several parameter queries
//   (e.g. manufacturer, serial, version etc).
//   To produce a hash from these, use {$DEFINE SHA256_HASH} which requires DCPsha256 in
//   David Bartones DCPcrypt is installed.
//  2. Does not require administrator priveleges to function.
//  3. Expecting to move other OS ID info into this module at a latter stage rather than ogutils

 {$IFDEF WINDOWS}
 function QueryWMI( QueryID : TID_Type; var OutBuf : AnsiString ) : Boolean;
 {$ENDIF}
//=============================================================
implementation

uses
{$IFDEF WINDOWS}
Windows, SysUtils, ActiveX,  ComObj,  Variants, // only Windows WMI
{$ENDIF}
Classes,    // only used for a TStringList.
ogutil;     // Has hashing functions.



//-------------------------------------------------------------
// Tries to extract the serial number from specified IDE disk.
//
// Parameters:
//   ControllerNumber - SCSI port number of the controller.
//   DriveNumber - SCSI port number of the controller.
// Notes:
//  1. The parameter ControllerNumber is ignored on Windows 9x/ME platforms and should be 0.
//  2. This function CAN NOT extract SCSI disk serial number.
//  {SR 20-10-2014}
{$IFDEF WINDOWS}
{*
This is slow but reliable and doesn't require Administrator priveleges.
To improve performance, we should look at creating a WMI monitor service for ourselves.
*}
function QueryWMI( QueryID : TID_Type; var OutBuf : AnsiString ) : Boolean;
const
  WbemUser            ='';
  WbemPassword        ='';
  WbemComputer        ='localhost';
  wbemFlagForwardOnly = $00000020;
  wbemFlagReturnImmediately  = $00000010;
var
  FSWbemLocator : OLEVariant;
  FWMIService   : OLEVariant;
  FWbemObjectSet: OLEVariant;
  FWbemObject   : OLEVariant;
  oEnum         : IEnumvariant;
  iValue        : LongWord;
  Query         : String;
  fields : TStringList ;
  a:Integer;
begin;
      fields:=TStringlist.Create;
      OutBuf := '';
      case QueryID of
              BIOS:  Begin fields.CommaText := 'Manufacturer,Version,SerialNumber,SoftwareElementID';     Query := 'CIM_BIOSElement'; End;
              DISK:  Begin fields.CommaText := 'Manufacturer,SerialNumber,PNPDeviceID';                   Query := 'Win32_DiskDrive'; End;
              MOBO:  Begin fields.CommaText := 'Caption,IdentifyingNumber,Name,UUID,Vendor,Version';      Query := 'Win32_ComputerSystemProduct'; End;
              OS  :  Begin fields.CommaText := 'Caption,CSName,InstallDate,Version';                      Query := 'CIM_OperatingSystem'; End;
              USER:  Begin fields.CommaText := 'SID';                                                     Query := 'Win32_UserProfile WHERE Loaded=TRUE and LocalPath LIKE "%Users%"'; End;
              //SYS:   Begin fields.CommaText := 'Manufacturer,Version,SerialNumber,SoftwareElementID';     Query := 'CIM_BIOSElement'; End;
      else
          exit;
      end;//case

  Query :=  'SELECT ' +fields.CommaText +' FROM ' +  Query;
  try
     CoInitialize(nil);
     FSWbemLocator := CreateOleObject('WbemScripting.SWbemLocator');
     FWMIService   := FSWbemLocator.ConnectServer(WbemComputer, 'root\CIMV2', WbemUser, WbemPassword);
     FWbemObjectSet:= FWMIService.ExecQuery(OLEVariant(Query),'WQL',wbemFlagForwardOnly);
     oEnum         := IUnknown(FWbemObjectSet._NewEnum) as IEnumVariant;
     while oEnum.Next(1, FWbemObject, iValue) = 0 do
           begin
                for a:=0 to fields.Count-1 do Begin
                       OutBuf:=  OutBuf + trim(string(FWbemObject.Properties_.Item(OLEVariant(fields[a])).Value) );
                end;
                result := OutBuf <>'';
                FWbemObject:=Unassigned;
           end;
  finally
   CoUninitialize;
   fields.Free;
  end;
end;
{$ENDIF}

end.



